@@ -53,6 +53,7 @@ pub fn tof64<T>(s: &[T]) -> Vec<f64> where T: Copy, f64: From<T> {
 }
 
 /// Necessary recast of a whole i64 slice to f64 
+/// 
 pub fn i64tof64(s: &[i64]) -> Vec<f64> {
     s.iter().map(| &x | x as f64).collect()
 }
@@ -172,6 +173,12 @@ pub trait Stats {
     /// Median and quartiles
     fn median(self) -> Result<Med>
         where Self: std::marker::Sized { bail!("median not implemented for this type")}
+    /// Fast median only by partitioning
+    fn newmedian(self) -> Result<f64> 
+        where Self: std::marker::Sized { bail!("newmedian not implemented for this type")}
+    /// MAD median absolute deviation: data spread estimator that is more stable than variance
+    fn mad(self) -> Result<f64> 
+        where Self: std::marker::Sized { bail!("median not implemented for this type")}    
     /// Zero median data, obtained by subtracting the median
     fn zeromedian(self) -> Result<Vec<f64>>
         where Self: std::marker::Sized { bail!("gwmeanstd not implemented for this type")}
@@ -246,8 +253,12 @@ pub trait Vecg<T,U> {
     fn jointpdf(self,v:&[U]) -> Vec<f64>;     
     /// Joint entropy of &[T],&[U] in nats (units of e)
     fn jointentropy(self, v:&[U]) -> f64;
-    /// Statistical pairwise dependence in the range [0,1] based on joint entropy
-    fn dependence(self, v:&[U]) -> f64; 
+    /// Statistical pairwise dependence of &[T] &[U] variables in the range [0,1] 
+    fn dependence(self, v:&[U]) -> f64;   
+    /// Statistical pairwise independence in the range [1,2] based on joint entropy
+    fn independence(self, v:&[U]) -> f64; 
+    /// Cholesky decomposition of positive definite matrix into LLt
+    // fn cholesky(self) -> Vec<f64>;
     /// Pearson's correlation.  
     fn correlation(self, v:&[U]) -> f64;
     /// Median based correlation
@@ -326,8 +337,10 @@ pub trait Vecu8 {
     fn jointpdfu8(self, v:&[u8]) -> Vec<Vec<u32>>;
     /// Joint entropy of &[u8],&[u8] (in nats)
     fn jointentropyu8(self, v:&[u8]) -> f64;
-    /// Dependence in the range [0,1] of two &[u8] variables 
-    fn dependenceu8(self, v:&[u8]) -> f64;
+    /// Statistical pairwise dependence of two &[u8] variables in the range [0,1] 
+    fn dependenceu8(self, v:&[u8]) -> f64;   
+    /// Independence in the range [1,2] of two &[u8] variables 
+    fn independenceu8(self, v:&[u8]) -> f64;
 }
 
 /// A few specialised methods applicable to `Vec<Vec<u8>>` (vector of vectors of bytes).
@@ -358,8 +371,10 @@ pub trait VecVec<T> {
     fn jointpdfn(self) -> Vec<f64>;
     /// Joint entropy between a set of vectors of the same length
     fn jointentropyn(self) -> f64;
-    /// Dependence of a set of vectors 
-    fn dependencen(self) -> f64;      
+    /// Independence (component wise) of a set of vectors.
+    fn dependencen(self) -> f64; 
+    /// Flattened lower triangular relations matrix between columns of self 
+    fn crossfeatures<F>(self,f:F) -> Vec<f64> where F: Fn(&[T],&[T]) -> f64; 
     /// Arithmetic Centre = euclidian mean of a set of points
     fn acentroid(self) -> Vec<f64>;
     /// Geometric Centroid
@@ -377,7 +392,7 @@ pub trait VecVec<T> {
     /// Next approx gm computed at a member point given by its indx
     fn nxmember(self, indx: usize) -> Vec<f64>;
     /// Next approx gm computed at a nonmember point p
-    fn nxnonmember(self, p:&[f64]) -> Vec<f64>;
+    fn nxnonmember(self, p:&[f64]) -> Vec<f64>; 
     /// Estimated ecentricity vector from a member point given by its indx to gm
     fn eccmember(self, indx: usize) -> Vec<f64>;
     /// Estimated eccentricity vector from a non member point
@@ -390,6 +405,8 @@ pub trait VecVec<T> {
     fn exacteccs(self, eps: f64) -> Vec<Vec<f64>>; 
     /// Median and quartiles of eccentricities (new robust measure of spread of a multivariate sample)
     fn eccinfo(self, eps: f64) -> (MStats,Med,MinMax<f64>) where Vec<f64>:FromIterator<f64>;
+    /// MADn multidimensional median absolute deviation: data spread estimator that is more stable than variance
+    fn madn(self, eps: f64) -> f64;
     /// Medoid and Outlier as defined by eccentricities.
     fn emedoid(self, eps: f64) -> MinMax<f64> where Vec<f64>:FromIterator<f64>;
     /// Returns sorted eccentricities magnitudes
@@ -438,9 +455,9 @@ pub trait VecVecg<T,U> {
     /// Flattened lower triangular part of a covariance matrix of a Vec of f64 vectors.
     fn covar(self, med:&[U]) -> Vec<f64>; 
     /// Flattened lower triangular part of a comediance matrix of a Vec of f64 vectors.
-    fn comed(self, m:&[U], eps:f64) -> Vec<f64>;
+    fn comed(self, m:&[U]) -> Vec<f64>;
     /// Flattened lower triangular part of a covariance matrix for weighted f64 vectors.
     fn wcovar(self, ws:&[U], m:&[f64]) -> Vec<f64>;
     /// Flattened lower triangular part of a comediance matrix for weighted f64 vectors.
-    fn wcomed(self, ws:&[U], m:&[f64], eps:f64) -> Vec<f64>;
+    fn wcomed(self, ws:&[U], m:&[f64]) -> Vec<f64>;
 }
