@@ -353,7 +353,7 @@ impl<T> Stats for &[T]
         })
     }
 
-    /// Median of a &[T] slice
+    /// Median of a &[T] slice by sorting
     /// # Example
     /// ```
     /// use rstats::{Stats};
@@ -393,6 +393,24 @@ impl<T> Stats for &[T]
         }
         Ok(result)       
     }
+/*
+    /// Fast median only of a &[T] slice by partitioning
+    fn newmedian(self) -> Result<f64> {
+        // define balance of signs closure
+        let balance = |x:f64| self.iter().map(|&s| (f64::from(s)-x).signum()).sum::<f64>(); 
+        let MinMax{min,minindex,max,maxindex} = minmax(self);        
+        Ok(1.)       
+    }
+*/
+    /// MAD median absolute deviation: data spread estimator that is more stable than variance
+    fn mad(self) -> Result<f64> {
+        let Med{median,..} = self.median() // ignore quartile fields
+            .unwrap_or_else(|_|panic!("{} failed to obtain median",here!()));
+        let diffs:Vec<f64> = self.iter().map(|&s| (f64::from(s)-median).abs()).collect();
+        let Med{median:res,..} = diffs.median()
+            .unwrap_or_else(|_|panic!("{} failed to obtain median",here!()));
+        Ok(res)
+    }
 
     /// Zero median data produced by subtracting the median.
     /// Analogous to zero mean data when subtracting the mean.
